@@ -2,6 +2,7 @@
 
 new_defaults = function(value = list()) {
   defaults = value
+  locked = FALSE
 
   get = function(name, default = FALSE, drop = TRUE) {
     if (default) defaults = value  # this is only a local version
@@ -18,10 +19,23 @@ new_defaults = function(value = list()) {
       if (length(dots <- dots[[1]]) == 0) return()
     dots
   }
+  set2 = function(values) {
+    old = get(names(values), drop = FALSE)
+    if (length(values)) {
+      # TODO: change warning() to stop() and no longer whitelist JuliaCall
+      if (locked && !identical(names(values), 'Jfig.cur')) warning(
+        'The object is read-only and cannot be modified. If you have to modify it ',
+        'for a legitimate reason, call the method $lock(FALSE) on the object before $set(). ',
+        'Using $lock(FALSE) to modify the object will be enforced in future versions of knitr ',
+        'and this warning will become an error.',
+        immediate. = TRUE
+      )
+      defaults <<- merge(values)
+    }
+    invisible(old)
+  }
   set = function(...) {
-    dots = resolve(...)
-    if (length(dots)) defaults <<- merge(dots)
-    invisible(NULL)
+    set2(resolve(...))
   }
   delete = function(keys) {
     for (k in keys) defaults[[k]] <<- NULL
@@ -31,13 +45,13 @@ new_defaults = function(value = list()) {
   append = function(...) {
     dots = resolve(...)
     for (i in names(dots)) dots[[i]] <- c(defaults[[i]], dots[[i]])
-    if (length(dots)) defaults <<- merge(dots)
-    invisible(NULL)
+    set2(dots)
   }
+  lock = function(status = TRUE) locked <<- status
 
   list(
-    get = get, set = set, delete = delete,
-    append = append, merge = merge, restore = restore
+    get = get, set = set, delete = delete, append = append, merge = merge,
+    restore = restore, lock = lock
   )
 }
 
@@ -61,9 +75,8 @@ new_defaults = function(value = list()) {
 #'   A list of available options:
 #'   \url{https://yihui.org/knitr/options/#chunk-options}
 #' @note \code{opts_current} should be treated as read-only and you are supposed
-#'   to only query its values via \code{opts_current$get()}. Technically you
-#'   could also call \code{opts_current$set()} to change the values, but you are
-#'   not recommended to do so unless you understand the consequences.
+#'   to only query its values via \code{opts_current$get()}. Calling
+#'   \code{opts_current$set()} will throw an error.
 #' @export
 #' @examples opts_chunk$get('prompt'); opts_chunk$get('fig.keep')
 opts_chunk = new_defaults(list(
@@ -93,7 +106,6 @@ opts_chunk = new_defaults(list(
 ))
 
 #' @rdname opts_chunk
-#' @format
 #' @export
 opts_current = new_defaults()
 opts_current$restore(opts_chunk$get())
@@ -115,16 +127,32 @@ opts_chunk_attr = local({
     opts$dev = grep('^quartz_', opts$dev, value = TRUE, invert = TRUE)
   if (.Platform$OS.type != 'windows')
     opts$dev = setdiff(opts$dev, 'win.metafile')
-  opts$dev = as.list(opts$dev)
+  opts$dev = opts$fig.format = as.list(opts$dev)
+  opts$fig.dpi = 'numeric'
   opts$fig.ext = as.list(unique(auto_exts))
   opts$external = opts$sanitize = NULL  # hide these two rare options
   opts$fig.process = 'function'
-  opts$fig.asp = 'numeric'
-  opts$fig.alt = 'character'
+  opts[c('fig.asp', 'fig.ncol')] = 'numeric'
   opts$fig.dim = 'list'
+  opts$fig.id = 'logical'
+  opts[c(
+    'opts.label', 'resize.width', 'resize.height', 'fig.alt', 'fig.link', 'fig.sep',
+    'tab.cap', 'ffmpeg.bitrate', 'ffmpeg.format'
+  )] = 'character'
+  opts$lang = 'list'
   opts$R.options = 'list'
   opts$cache.comments = 'logical'
+  opts$cache.globals = 'list'
   opts$animation.hook = list('ffmpeg', 'gifski')
+  for (i in c('class', 'attr')) {
+    for (j in c('source', 'output', 'message', 'warning', 'error', 'chunk')) {
+      opts[[paste(i, j, sep = '.')]] = 'character'
+    }
+  }
+  # for R Markdown paged tables
+  opts[paste0(c(
+    'max', 'sql.max', 'paged', 'rows', 'cols', 'cols.min', 'pages', 'paged', 'rownames'
+  ), '.print')] = 'numeric'
   opts
 })
 
@@ -160,7 +188,7 @@ set_alias = function(...) {
 #' @references Usage: \url{https://yihui.org/knitr/objects/}
 #'
 #'   A list of available options:
-#'   \url{https://yihui.org/knitr/options/#package_options}
+#'   \url{https://yihui.org/knitr/options/#package-options}
 #' @export
 #' @examples opts_knit$get('verbose'); opts_knit$set(verbose = TRUE)  # change it
 #' if (interactive()) {
@@ -170,7 +198,7 @@ set_alias = function(...) {
 #' }
 #' @include hooks-html.R
 opts_knit = new_defaults(list(
-  progress = TRUE, verbose = FALSE, eval.after = c('fig.cap', 'fig.alt'),
+  progress = TRUE, verbose = FALSE, eval.after = c('fig.cap', 'fig.scap', 'fig.alt'),
   base.dir = NULL, base.url = NULL, root.dir = NULL, child.path = '',
   upload.fun = identity, global.device = FALSE, global.par = FALSE,
   concordance = FALSE, documentation = 1L, self.contained = TRUE,
