@@ -13,12 +13,12 @@ insert_header = function(doc) {
 }
 
 # Makes latex header with macros required for highlighting, tikz and framed
-make_header_latex = function() {
+make_header_latex = function(doc) {
   h = one_string(c(
-    sprintf('\\usepackage[%s]{graphicx}\\usepackage[%s]{color}',
-            opts_knit$get('latex.options.graphicx') %n% '',
-            opts_knit$get('latex.options.color') %n% ''),
+    header_latex_packages(doc),
     .header.maxwidth, opts_knit$get('header'),
+    if (length(grep('\\hlstd', doc, fixed = TRUE))) '\\let\\hlstd\\hldef',
+    if (length(grep('\\hlstr', doc, fixed = TRUE))) '\\let\\hlstr\\hlsng',
     if (getOption('OutDec') != '.') '\\usepackage{amsmath}',
     if (out_format('latex')) '\\usepackage{alltt}'
   ))
@@ -28,6 +28,24 @@ make_header_latex = function() {
   }
 }
 
+# if the document already contains \usepackage[options]{pkg}, use the same
+# options to avoid the option clash error in LaTeX
+use_package = function(pkg, doc) {
+  opts = sapply(pkg, function(p) {
+    r = sprintf('.*?\\\\usepackage\\[(.+?)]\\{%s}.*', p)
+    o = xfun::grep_sub(r, '\\1', doc)
+    if (length(o)) return(o[1])
+    opts_knit$get(paste0('latex.options.', p)) %n% ''
+  })
+  sprintf('\\usepackage[%s]{%s}', opts, pkg)
+}
+
+# for backward-compatibility, use xcolor package unless the latex.options.color
+# has been set; xcolor is preferred: https://github.com/latex3/latex2e/pull/719
+header_latex_packages = function(doc) {
+  paste(use_package(c('graphicx', 'xcolor'), doc), collapse = '')
+}
+
 insert_header_latex = function(doc, b) {
   i = grep(b, doc)
   if (length(i) >= 1L) {
@@ -39,13 +57,12 @@ insert_header_latex = function(doc, b) {
       j = j[1]
       doc[j] = sub(p, '\n\\\\IfFileExists{upquote.sty}{\\\\usepackage{upquote}}{}\n\\2', doc[j], perl = TRUE)
     }
-    i = i[1L]; l = stringr::str_locate(doc[i], b)
-    tmp = stringr::str_sub(doc[i], l[, 1], l[, 2])
-    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, make_header_latex())
+    i = i[1L]; l = str_locate(doc[i], b, FALSE)
+    doc[i] = str_insert(doc[i], l[, 2], make_header_latex(doc))
   } else if (parent_mode() && !child_mode()) {
     # in parent mode, we fill doc to be a complete document
     doc[1L] = one_string(c(
-      getOption('tikzDocumentDeclaration'), make_header_latex(),
+      getOption('tikzDocumentDeclaration'), make_header_latex(doc),
       .knitEnv$tikzPackages, '\\begin{document}', doc[1L]
     ))
     doc[length(doc)] = one_string(
@@ -70,9 +87,8 @@ make_header_html = function() {
 insert_header_html = function(doc, b) {
   i = grep(b, doc)
   if (length(i) == 1L) {
-    l = stringr::str_locate(doc[i], b)
-    tmp = stringr::str_sub(doc[i], l[, 1], l[, 2])
-    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, '\n', make_header_html())
+    l = str_locate(doc[i], b, FALSE)
+    doc[i] = str_insert(doc[i], l[, 2], paste0('\n', make_header_html()))
   }
   doc
 }
